



----------------------------------------------------------------------------
--  This file is a part of the LEON VHDL model
--  Copyright (C) 1999  European Space Agency (ESA)
--
--  This library is free software; you can redistribute it and/or
--  modify it under the terms of the GNU Lesser General Public
--  License as published by the Free Software Foundation; either
--  version 2 of the License, or (at your option) any later version.
--
--  See the file COPYING.LGPL for the full details of the license.


-----------------------------------------------------------------------------   
-- Entity:      dcache
-- File:        dcache.vhd
-- Author:      Jiri Gaisler - Gaisler Research
-- Description: This unit implements the data cache controller.
------------------------------------------------------------------------------  

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned."+";
use IEEE.std_logic_unsigned.conv_integer;
use IEEE.std_logic_arith.conv_unsigned;
use work.amba.all;
use work.target.all;
use work.config.all;
use work.sparcv8.all;		-- ASI declarations
use work.iface.all;
use work.macro.all;		-- xorv()

entity dcache is
    port (
        rst             : in  std_logic;
        clk             : in  clk_type;
        dci             : in  dcache_in_type;
        dco             : out dcache_out_type;
        ico             : in  icache_out_type;
        mcdi            : out memory_dc_in_type;
        mcdo            : in  memory_dc_out_type;
        ahbsi           : in  ahb_slv_in_type;
    --BSM:Step2: dcrami : out dcram_in_type;
    --BSM:Step2: dcramo : in  dcram_out_type;
        fpuholdn        : in  std_logic
    );
end; 

architecture rtl of dcache is

constant TAG_HIGH       : integer := DTAG_HIGH;
constant TAG_LOW        : integer := DOFFSET_BITS + DLINE_BITS + 2;
constant OFFSET_HIGH    : integer := TAG_LOW - 1;
constant OFFSET_LOW     : integer := DLINE_BITS + 2;
constant LINE_HIGH      : integer := OFFSET_LOW - 1;
constant LINE_LOW       : integer := 2;
constant LINE_ZERO      : std_logic_vector(DLINE_BITS-1 downto 0) := (others => '0');
constant SETBITS        : integer := log2x(DSETS); 

type rdatatype is (dtag, ddata, icache, memory);  -- sources during cache read
--BSM:Step2a: type vmasktype is (clearone, clearall, merge, tnew);	-- valid bits operation

type write_buffer_type is record			-- write buffer 
    addr, data1, data2             : std_logic_vector(31 downto 0);
    size                           : std_logic_vector(1 downto 0);
    asi                            : std_logic_vector(3 downto 0);
    read                           : std_logic;
    lock                           : std_logic;
end record;

type dcache_control_type is record			-- all registers
    read                           : std_logic;					-- access direction
    signed                         : std_logic;					-- signed/unsigned read
    size                           : std_logic_vector(1 downto 0);	-- access size
    req, burst, holdn, nomds, stpend    : std_logic;
    xaddress                       : std_logic_vector(31 downto 0);-- common address buffer
    --BSM:Step2c: faddr                          : std_logic_vector(DOFFSET_BITS - 1 downto 0);	-- flush address
    --BSM:Step2c: valid                          : std_logic_vector(DLINE_SIZE - 1 downto 0);	-- registered valid bits
    dstate                         : std_logic_vector(2 downto 0);	-- FSM vector
    --BSM:Step2c: hit                            : std_logic;
    flush		                   : std_logic;				    -- flush in progress
    mexc 		                   : std_logic;				    -- latched mexc
    wb 		                       : write_buffer_type;			-- write buffer
    asi  		                   : std_logic_vector(3 downto 0);
    icenable	                   : std_logic;				    -- icache diag access
    --BSM:Step2b: rndcnt                         : std_logic_vector(log2x(DSETS)-1 downto 0); -- replace counter
    --BSM:Step2b: setrepl                        : std_logic_vector(log2x(DSETS)-1 downto 0); -- set to replace
  --BSM:Step2a: lrr           : std_logic;            
    --BSM:Step2c: dsuset                         : std_logic_vector(log2x(DSETS)-1 downto 0);
    lock                           : std_logic;
end record;

--BSM:Step2b: type snoop_reg_type is record			-- snoop control registers
    --BSM:Step2b: snoop                          : std_logic;				-- snoop access to tags
    --BSM:Step2b: writebp                        : std_logic_vector(0 to DSETS-1);		-- snoop write bypass
    --BSM:Step2b: addr 	                       : std_logic_vector(TAG_HIGH downto OFFSET_LOW);-- snoop tag
--BSM:Step2b: end record;

--BSM:Step2b: type snoop_hit_bits_type is array (0 to 2**DOFFSET_BITS-1) of std_logic_vector(0 to DSETS-1);

--BSM:Step2b: type snoop_hit_reg_type is record
    --BSM:Step2b: hit 	                       : snoop_hit_bits_type;           -- snoop hit bits  
    --BSM:Step2b: taddr	                       : std_logic_vector(OFFSET_HIGH downto OFFSET_LOW);  -- saved tag address
    --BSM:Step2b: set                            : std_logic_vector(log2x(DSETS)-1 downto 0);        -- saved set
--BSM:Step2b: end record;

--BSM:Step2a: subtype lru_type is std_logic_vector(DLRUBITS-1 downto 0);
--BSM:Step2a: type lru_array  is array (0 to 2**DOFFSET_BITS-1) of lru_type;  -- lru registers
--BSM:Step2a: type par_type is array (0 to ISETS-1) of std_logic_vector(1 downto 0);

--BSM:Step2a: type lru_reg_type is record
  --BSM:Step2a: write : std_logic;
  --BSM:Step2a: waddr : std_logic_vector(DOFFSET_BITS-1 downto 0);
  --BSM:Step2a: set   :  std_logic_vector(SETBITS-1 downto 0); --integer range 0 to DSETS-1;
  --BSM:Step2a: lru   : lru_array;
--BSM:Step2a: end record;


--BSM:Step2a: subtype lock_type is std_logic_vector(0 to DSETS-1);

--BSM:Step2a: function lru_set (lru : lru_type; lock : lock_type) return std_logic_vector is
--BSM:Step2a: variable xlru : std_logic_vector(4 downto 0);
--BSM:Step2a: variable set  : std_logic_vector(SETBITS-1 downto 0);
--BSM:Step2a: variable xset : std_logic_vector(1 downto 0);
--BSM:Step2a: variable unlocked : integer range 0 to DSETS-1;
--BSM:Step2a: begin
  --BSM:Step2a: set := (others => '0'); xlru := (others => '0');
  --BSM:Step2a: xlru(DLRUBITS-1 downto 0) := lru;

  --BSM:Step2a: if DCLOCK_BIT = 1 then 
    --BSM:Step2a: unlocked := DSETS-1;
    --BSM:Step2a: for i in DSETS-1 downto 0 loop
      --BSM:Step2a: if lock(i) = '0' then
        --BSM:Step2a: unlocked := i;
      --BSM:Step2a: end if;
    --BSM:Step2a: end loop;
  --BSM:Step2a: end if;

  --BSM:Step2a: case DSETS is
      --BSM:Step2a: when 2 =>
        --BSM:Step2a: if DCLOCK_BIT = 1 then
          --BSM:Step2a: if lock(0) = '1' then
            --BSM:Step2a: xset(0) := '1'; 
          --BSM:Step2a: else 
            --BSM:Step2a: xset(0) := xlru(0);
          --BSM:Step2a: end if;
        --BSM:Step2a: else
          --BSM:Step2a: xset(0) := xlru(0);
        --BSM:Step2a: end if;
      --BSM:Step2a: when 3 => 
        --BSM:Step2a: if DCLOCK_BIT = 1 then
          --BSM:Step2a: xset := std_logic_vector(conv_unsigned(lru3_repl_table(conv_integer(xlru)) (unlocked), 2));
        --BSM:Step2a: else
          --BSM:Step2a: xset := std_logic_vector(conv_unsigned(lru3_repl_table(conv_integer(xlru)) (0), 2));
        --BSM:Step2a: end if;
      --BSM:Step2a: when 4 =>
        --BSM:Step2a: if DCLOCK_BIT = 1 then
          --BSM:Step2a: xset := std_logic_vector(conv_unsigned(lru4_repl_table(conv_integer(xlru)) (unlocked), 2));
        --BSM:Step2a: else
          --BSM:Step2a: xset := std_logic_vector(conv_unsigned(lru4_repl_table(conv_integer(xlru)) (0), 2));
        --BSM:Step2a: end if;    
      --BSM:Step2a: when others => 
  --BSM:Step2a: end case;
  --BSM:Step2a: set := xset(SETBITS-1 downto 0);
  --BSM:Step2a: return(set);
--BSM:Step2a: end;

--BSM:Step2a: function lru_calc (lru : lru_type; set : integer) return lru_type is
--BSM:Step2a: variable new_lru : lru_type;
--BSM:Step2a: variable xnew_lru: std_logic_vector(4 downto 0);
--BSM:Step2a: variable xlru : std_logic_vector(4 downto 0);
--BSM:Step2a: begin
  --BSM:Step2a: new_lru := (others => '0'); xnew_lru := (others => '0');
  --BSM:Step2a: xlru := (others => '0'); xlru(DLRUBITS-1 downto 0) := lru;
  --BSM:Step2a: case DSETS is
  --BSM:Step2a: when 2 => 
    --BSM:Step2a: if set = 0 then xnew_lru(0) := '1'; else xnew_lru(0) := '0'; end if;
  --BSM:Step2a: when 3 =>
    --BSM:Step2a: xnew_lru(2 downto 0) := lru_3set_table(conv_integer(lru))(set); 
  --BSM:Step2a: when 4 => 
    --BSM:Step2a: xnew_lru(4 downto 0) := lru_4set_table(conv_integer(lru))(set);
  --BSM:Step2a: when others => 
  --BSM:Step2a: end case;
  --BSM:Step2a: new_lru := xnew_lru(DLRUBITS-1 downto 0);
  --BSM:Step2a: return(new_lru);
--BSM:Step2a: end;


signal r, c : dcache_control_type;	-- r is registers, c is combinational
--BSM:Step2b: signal rs, cs : snoop_reg_type;		-- rs is registers, cs is combinational
--BSM:Step2b: signal rh, ch : snoop_hit_reg_type;	-- rs is registers, cs is combinational
--BSM:Step2a: signal rl, cl : lru_reg_type;           -- rl is registers, cl is combinational


begin

  --BSM:Step2: dctrl : process(rst, r, rs, rh, rl, dci, mcdo, ico, dcramo, ahbsi, fpuholdn)
  --BSM:Step2a: dctrl : process(rst, r, rs, rh, rl, dci, mcdo, ico, ahbsi, fpuholdn)
    --BSM:Step2b: dctrl : process(rst, r, rs, rh, dci, mcdo, ico, ahbsi, fpuholdn)
    dctrl : process(rst, r, dci, mcdo, ico, ahbsi, fpuholdn)
  --BSM:Step2: variable dcramov : dcram_out_type;
    variable rdatasel : rdatatype;
    variable maddress : std_logic_vector(31 downto 0);
    variable maddrlow : std_logic_vector(1 downto 0);
    variable edata : std_logic_vector(31 downto 0);
    variable size : std_logic_vector(1 downto 0);
    variable read : std_logic;
  --BSM:Step2a: variable twrite, tdiagwrite, ddiagwrite, dwrite : std_logic;
  --BSM:Step2a: variable taddr : std_logic_vector(OFFSET_HIGH  downto LINE_LOW); -- tag address
  --BSM:Step2a: variable newtag : std_logic_vector(TAG_HIGH  downto TAG_LOW); -- new tag
    variable align_data : std_logic_vector(31 downto 0); -- aligned data
  --BSM:Step2a: variable ddatain : std_logic_vector(31 downto 0);
    variable rdata : std_logic_vector(31 downto 0);

  --BSM:Step2a: variable vmaskraw, vmask : std_logic_vector((DLINE_SIZE -1) downto 0);
  --BSM:Step2a: variable vmaskdbl : std_logic_vector((DLINE_SIZE/2 -1) downto 0);
    variable enable : std_logic;
    variable mds : std_logic;
    variable mexc : std_logic;
    --BSM:Step2c: variable hit, valid, validraw, forcemiss : std_logic;
    variable signed   : std_logic;
    --BSM:Step2c: variable flush    : std_logic;
    variable iflush   : std_logic;
    variable v : dcache_control_type;
    variable eholdn : std_logic;				-- external hold
    --BSM:Step2c: variable tparerr, dparerr  : std_logic_vector(0 to ISETS-1);
    --BSM:Step2c: variable snoopwe  : std_logic;
    --BSM:Step2c: variable hcache   : std_logic;
    --BSM:Step2b: variable snoopaddr: std_logic_vector(OFFSET_HIGH downto OFFSET_LOW);
    --BSM:Step2b: variable vs : snoop_reg_type;
    --BSM:Step2b: variable vh : snoop_hit_reg_type;
    variable dsudata   : std_logic_vector(31 downto 0);
    --BSM:Step2c: variable set : integer range 0 to DSETS-1;
    --BSM:Step2c: variable snoopset : integer range 0 to DSETS-1;
    --BSM:Step2c: variable csnoopwe : std_logic_vector(0 to MAXSETS-1);
  --BSM:Step2a: variable ctwrite, cdwrite : std_logic_vector(0 to MAXSETS-1);
    --BSM:Step2b: variable setrepl  : std_logic_vector(log2x(DSETS)-1 downto 0);
  --BSM:Step2a: variable wlrr : std_logic_vector(0 to MAXSETS-1);
  --BSM:Step2a: variable vl : lru_reg_type;
  --BSM:Step2a: variable diagset : std_logic_vector(TAG_LOW + SETBITS -1 downto TAG_LOW);
  --BSM:Step2a: variable lock : std_logic_vector(0 to DSETS-1);
  --BSM:Step2a: variable wlock : std_logic_vector(0 to MAXSETS-1);
    --BSM:Step2c: variable snoopset2, rdsuset : integer range 0 to DSETS-1;

    begin

-- init local variables

    v := r;
    --BSM:Step2b: vs := rs;
    --BSM:Step2b: vh := rh; 
    --BSM:Step2: dcramov := dcramo;
    --BSM:Step2a: vl := rl;
    --BSM:Step2a: vl.write := '0'; 

    mds := '1';
    --BSM:Step2a: dwrite := '0';
    --BSM:Step2a: twrite := '0'; 
    --BSM:Step2a: ddiagwrite := '0'; tdiagwrite := '0';
    v.holdn := '1';
    mexc := '0';
    --BSM:Step2c: flush := '0';
    v.icenable := '0';
    iflush := '0';
    eholdn := ico.hold and fpuholdn;
    --BSM:Step2c: tparerr  := (others => '0');
    --BSM:Step2c: dparerr  := (others => '0'); 
    --BSM:Step2b: vs.snoop := '0';
    --BSM:Step2b: vs.writebp := (others => '0');
    --BSM:Step2c: snoopwe := '0';
    --BSM:Step2b: snoopaddr := ahbsi.haddr(OFFSET_HIGH downto OFFSET_LOW);
    --BSM:Step2c: hcache := '0';
    --BSM:Step2c: rdsuset := 0;
    enable := '1';

    rdatasel := ddata;	-- read data from cache as default

    --BSM:Step2c: set := 0;
    --BSM:Step2c: snoopset := 0;
    --BSM:Step2c: csnoopwe := (others => '0');
    --BSM:Step2a: ctwrite := (others => '0'); cdwrite := (others => '0');
    --BSM:Step2a: wlock := (others => '0');
    --BSM:Step2: for i in 0 to DSETS-1 loop wlock(i) := dcramov.dtramout(i).lock; end loop; 
    --BSM:Step2a: wlrr := (others => '0');
    --BSM:Step2: for i in 0 to 1 loop wlrr(i) := dcramov.dtramout(i).lrr; end loop; 
    
    --BSM:Step2a: if (DSETS > 1) then
        --BSM:Step2a: setrepl := r.setrepl;
    --BSM:Step2a: else
        --BSM:Step2b: setrepl := (others => '0');
    --BSM:Step2a: end if;
    
-- random replacement counter
    --BSM:Step2a: if DSETS > 1 then
-- pragma translate_off
      --BSM:Step2a: if not is_x(r.rndcnt) then
-- pragma translate_on
        --BSM:Step2a: if conv_integer(r.rndcnt) = (DSETS - 1) then
            --BSM:Step2a: v.rndcnt := (others => '0');
        --BSM:Step2a: else
            --BSM:Step2a: v.rndcnt := r.rndcnt + 1;
        --BSM:Step2a: end if;
-- pragma translate_off
      --BSM:Step2a: end if;
-- pragma translate_on
    --BSM:Step2a: end if;

-- generate lock bits
    --BSM:Step2: if DCLOCK_BIT = 1 then 
      --BSM:Step2: for i in 0 to DSETS-1 loop lock(i) := dcramov.dtramout(i).lock; end loop;
    --BSM:Step2: end if;
    
-- AHB snoop handling

    --BSM:Step2b: if DSNOOP then

      -- snoop only in cacheable areas
      --BSM:Step2b: for i in PROC_CACHETABLE'range loop	--'
        --BSM:Step2b: if (ahbsi.haddr(31 downto 32-PROC_CACHE_ADDR_MSB) >= PROC_CACHETABLE(i).firstaddr) and
           --BSM:Step2b: (ahbsi.haddr(31 downto 32-PROC_CACHE_ADDR_MSB) < PROC_CACHETABLE(i).lastaddr) 
        --BSM:Step2b: then
            --BSM:Step2b: hcache := '1';
        --BSM:Step2b: end if;
      --BSM:Step2b: end loop;

      -- snoop on NONSEQ or SEQ and first word in cache line
      -- do not snoop during own transfers or during cache flush
      --BSM:Step2b: if (ahbsi.hready and ahbsi.hwrite and not mcdo.bg) = '1' and
         --BSM:Step2b: ((ahbsi.htrans = HTRANS_NONSEQ) or 
	    --BSM:Step2b: ((ahbsi.htrans = HTRANS_SEQ) and 
	     --BSM:Step2b: (ahbsi.haddr(LINE_HIGH downto LINE_LOW) = LINE_ZERO))) 
      --BSM:Step2b: then
	    --BSM:Step2b: vs.snoop := mcdo.dsnoop and hcache;
        --BSM:Step2b: vs.addr := ahbsi.haddr(TAG_HIGH downto OFFSET_LOW); 
      --BSM:Step2b: end if;
      -- clear valid bits on snoop hit (or set hit bits)
      --BSM:Step2b: for i in DSETS-1 downto 0 loop
        --BSM:Step2b: if ((rs.snoop and (not mcdo.ba) and not r.flush) = '1') 
          --BSM:Step2: and (dcramov.dtramoutsn(i).tag = rs.addr(TAG_HIGH downto TAG_LOW))  -- DSNOOP only
        --BSM:Step2b: then
          --BSM:Step2b: if DSNOOP_FAST then
-- pragma translate_off
            --BSM:Step2b: if not is_x(rs.addr(OFFSET_HIGH downto OFFSET_LOW)) then
-- pragma translate_on
                --BSM:Step2b: vh.hit(conv_integer(rs.addr(OFFSET_HIGH downto OFFSET_LOW)))(i) := '1';
--              vh.set := std_logic_vector(conv_unsigned(i, SETBITS));
-- pragma translate_off
            --BSM:Step2b: end if;
-- pragma translate_on
          --BSM:Step2b: else
            --BSM:Step2b: snoopaddr := rs.addr(OFFSET_HIGH downto OFFSET_LOW);
            --BSM:Step2b: snoopwe := '1'; snoopset := i;        
          --BSM:Step2b: end if;
        --BSM:Step2b: end if;
      -- bypass tag data on read/write contention
        --BSM:Step2: if (not DSNOOP_FAST) and (rs.writebp(i) = '1') then 
          --BSM:Step2: dcramov.dtramout(i).tag   := rs.addr(TAG_HIGH downto TAG_LOW);
          --BSM:Step2: dcramov.dtramout(i).valid := (others => '0');
        --BSM:Step2: end if;
      --BSM:Step2b: end loop;
    --BSM:Step2b: end if;   -- end AHB snoop handling --BSM:Step2a: 

-- generate access parameters during pipeline stall

    --BSM:Step2a: if ((r.holdn) = '0') or (DEBUG_UNIT and (dci.dsuen = '1')) then
      --BSM:Step2a: taddr := r.xaddress(OFFSET_HIGH downto LINE_LOW);
      --if r.dsuwren = '0' then v.dsuwren := '1'; end if;
    --BSM:Step2a: elsif ((dci.enaddr and not dci.read) = '1') or (eholdn = '0') then
      --BSM:Step2a: taddr := dci.maddress(OFFSET_HIGH downto LINE_LOW);
    --BSM:Step2a: else
      --BSM:Step2a: taddr := dci.eaddress(OFFSET_HIGH downto LINE_LOW);
    --BSM:Step2a: end if;

    if (dci.write or not r.holdn) = '1' then
        maddress := r.xaddress(31 downto 0);
        signed := r.signed; 
        read := r.read;
        size := r.size;
        edata := dci.maddress;
    else
        maddress := dci.maddress(31 downto 0);
        signed := dci.signed; 
        read := dci.read;
        size := dci.size;
        edata := dci.edata;
    end if;

    --BSM:Step2a: newtag := dci.maddress(TAG_HIGH downto TAG_LOW);
    --BSM:Step2a: vl.waddr := maddress(OFFSET_HIGH downto OFFSET_LOW);  -- lru write address

-- generate cache hit and valid bits
    --BSM:Step2c: forcemiss := not dci.asi(3);
    --BSM:Step2c: hit := '0';
    --BSM:Step2c: set := 0;
    --BSM:Step2: for i in DSETS-1 downto 0 loop
      --BSM:Step2: if (dcramov.dtramout(i).tag = dci.maddress(TAG_HIGH downto TAG_LOW)) then 
        --BSM:Step2: hit := ((not r.flush) and not tparerr(i)); set := i; 
      --BSM:Step2: end if;
    --BSM:Step2: end loop;


    --BSM:Step2a: if (dci.dsuen and (not r.holdn)) = '1' then
        --BSM:Step2a: diagset := r.xaddress(TAG_LOW+SETBITS-1 downto TAG_LOW);
    --BSM:Step2a: else
        --BSM:Step2a: diagset := maddress(TAG_LOW + SETBITS - 1 downto TAG_LOW);
    --BSM:Step2a: end if;
-- pragma translate_off
    --BSM:Step2a: if not is_x(diagset) then
-- pragma translate_on
      --BSM:Step2a: if ((dci.asi(3 downto 0) = ASI_DTAG) or (dci.asi(3 downto 0) = ASI_DDATA)) then
        --BSM:Step2a: case DSETS is
        --BSM:Step2a: when 1 =>
            --BSM:Step2a: set := 0;
        --BSM:Step2a: when 3 =>
            --BSM:Step2a: if conv_integer(diagset) < 3 then
                --BSM:Step2a: set := conv_integer(diagset);
            --BSM:Step2a: end if;
        --BSM:Step2a: when others => 
            --BSM:Step2a: set := conv_integer(diagset); 
        --BSM:Step2a: end case;
      --BSM:Step2a: end if;
-- pragma translate_off
    --BSM:Step2a: end if;
--pragma translate_on

--     if (dci.dsuen and (not r.holdn)) = '1' then
--       set := r.xaddress(TAG_LOW+SETBITS-1 downto TAG_LOW);
--     end if;
    
    
-- force miss on snoop hit

    --BSM:Step2b: if DSNOOP and DSNOOP_FAST then
-- pragma translate_off
      --BSM:Step2b: if not is_x(rh.taddr) then
-- pragma translate_on        
        --BSM:Step2b: hit := hit and not (rh.hit(conv_integer(rh.taddr))(conv_integer(rh.set)));
-- pragma translate_off
      --BSM:Step2b: end if;
-- pragma translate_on
    --BSM:Step2b: end if;

    --BSM:Step2: validraw := genmux(dci.maddress(LINE_HIGH downto LINE_LOW),
		    --BSM:Step2: dcramov.dtramout(set).valid);
	--BSM:Step2c: validraw := '0';
    --BSM:Step2c: valid := validraw and not dparerr(set);
    
    if ((r.holdn and dci.enaddr) = '1')  and (r.dstate = "000") then
        --BSM:Step2c: v.hit := hit;
        v.xaddress := dci.maddress;
	    v.read := dci.read;
        v.size := dci.size;
	    v.asi := dci.asi(3 downto 0); 
	    v.signed := dci.signed;
    end if;

-- Store buffer
    -- wdata := r.wb.data1;
    if mcdo.ready = '1' then
        v.wb.addr(2) := r.wb.addr(2) or (r.wb.size(0) and r.wb.size(1));
        if r.stpend = '1' then
            v.stpend := r.req;
            v.wb.data1 := r.wb.data2; 
	        v.wb.lock := r.wb.lock and r.req;
        end if;
    end if;
    if mcdo.grant = '1' then
        v.req := r.burst;
        v.burst := '0';
    end if;

-- main Dcache state machine
    case r.dstate is
        when "000" =>			-- Idle state
            v.nomds := r.nomds and not eholdn; 
          --BSM:Step2: v.valid := dcramov.dtramout(set).valid;
            --BSM:Step2c: v.valid := (others =>'0');
            if (r.stpend  = '0') or ((mcdo.ready and not r.req)= '1') then -- wait for store queue
    	        v.wb.addr := dci.maddress;
                v.wb.size := dci.size; 
    	        v.wb.read := dci.read;
                v.wb.data1 := dci.edata;
                v.wb.lock := dci.lock;
    	        v.wb.asi := dci.asi(3 downto 0); 
            end if;
            if (eholdn and (not r.nomds)) = '1' then -- avoid false path through nullify
    	        if dci.asi(3 downto 0) = ASI_DTAG then
                    rdatasel := dtag;
                end if;
            end if;
            if (dci.enaddr and eholdn and (not r.nomds) and not dci.nullify) = '1' then
    	    case dci.asi(3 downto 0) is
    	        when ASI_ITAG | ASI_IDATA =>		-- Read/write Icache tags
    	            if ico.flush = '1' then
                        mexc := '1';
     	            else
                        v.dstate := "101"; v.holdn := '0';
                    end if;
     	        when ASI_IFLUSH =>		-- flush instruction cache
    	            if dci.read = '0' then
                        iflush := '1';
                    end if;
     	        --BSM:Step2a: when ASI_DFLUSH =>		-- flush data cache
    	            --BSM:Step2a: if dci.read = '0' then
                        --BSM:Step2a: flush := '1';
                    --BSM:Step2a: end if;
     	        --BSM:Step2a: when ASI_DDATA =>		-- Read/write Dcache data
     	            --BSM:Step2a: if (dci.size /= "10") or (r.flush = '1') then -- only word access is allowed
     	                --BSM:Step2a: mexc := '1';
     	            --BSM:Step2a: elsif (dci.read = '0') then
     	                --BSM:Step2a: dwrite := '1'; ddiagwrite := '1';
     	            --BSM:Step2a: end if;
     	        --BSM:Step2a: when ASI_DTAG =>		-- Read/write Dcache tags
     	            --BSM:Step2a: if (dci.size /= "10") or (r.flush = '1') then -- allow only word access
     	                --BSM:Step2a: mexc := '1';
     	            --BSM:Step2a: elsif (dci.read = '0') then
     	                --BSM:Step2a: twrite := '1'; tdiagwrite := '1';
     	            --BSM:Step2a: end if;
    	        when others =>
    	            if dci.read = '1' then	-- read access
    	                --BSM:Step2c: if (not ((mcdo.dcs(0) = '1') and ((hit and valid and not forcemiss) = '1'))) then	-- read miss
    	                    if (not ((mcdo.dcs(0) = '1'))) then
                            v.holdn := '0'; v.dstate := "001";
    	                    if ((r.stpend  = '0') or ((mcdo.ready and not r.req) = '1')) then	-- wait for store queue
    	                        v.req := '1'; 
    	                        v.burst := dci.size(1) and dci.size(0) and not dci.maddress(2);
                            end if;
                        --BSM:Step2a: else       -- read hit
                            --BSM:Step2a: if (DSETS > 1) and (DCREPLACE = lru) then
                                --BSM:Step2a: vl.write := '1';
                            --BSM:Step2a: end if;
                        end if;
                
    	            else			-- write access
    	                if (r.stpend  = '0') or ((mcdo.ready and not r.req)= '1') then	-- wait for store queue
    	                    v.req := '1';
                            v.stpend := '1'; 
    	                    v.burst := dci.size(1) and dci.size(0);
    	                    if (dci.size = "11") then
                                v.dstate := "100";
                            end if; -- double store
    	                else		-- wait for store queue
    	                    v.dstate := "110";
                            v.holdn := '0';
    	                end if;
    	    
            --BSM:Step2a: if (mcdo.dcs(0) = '1') and ((hit and (dci.size(1) or validraw)) = '1') then  -- write hit
    	      --BSM:Step2a: twrite := '1'; dwrite := '1';
              --BSM:Step2a: if (DSETS > 1) and (DCREPLACE = lru) then
                --BSM:Step2a: vl.write := '1';
              --BSM:Step2a: end if;
              --BSM:Step2a: setrepl := std_logic_vector(conv_unsigned(set, SETBITS));
    	    --BSM:Step2a: end if;
    	    
                        if (dci.size = "11") then
                            v.xaddress(2) := '1';
                        end if;
    	            end if;
    
              --BSM:Step2a: if (DSETS > 1) then
        	    --BSM:Step2a: vl.set := std_logic_vector(conv_unsigned(set, SETBITS));
                --BSM:Step2a: v.setrepl := std_logic_vector(conv_unsigned(set, SETBITS));
                --BSM:Step2a: if ((not hit) and (not dparerr(set)) and (not r.flush)) = '1' then
                  --BSM:Step2a: case DCREPLACE is
                  --BSM:Step2a: when rnd =>
                    --BSM:Step2a: if DCLOCK_BIT = 1 then 
                      --BSM:Step2a: if lock(conv_integer(r.rndcnt)) = '0' then v.setrepl := r.rndcnt;
                      --BSM:Step2a: else
                        --BSM:Step2a: v.setrepl := std_logic_vector(conv_unsigned(DSETS-1, SETBITS));
                        --BSM:Step2a: for i in DSETS-1 downto 0 loop
                          --BSM:Step2a: if (lock(i) = '0') and (i>conv_integer(r.rndcnt)) then
                            --BSM:Step2a: v.setrepl := std_logic_vector(conv_unsigned(i, SETBITS));
                          --BSM:Step2a: end if;
                        --BSM:Step2a: end loop;
                      --BSM:Step2a: end if;
                    --BSM:Step2a: else
                      --BSM:Step2a: v.setrepl := r.rndcnt;
                    --BSM:Step2a: end if;
                  --BSM:Step2a: when lru =>
                    --BSM:Step2a: v.setrepl := lru_set(rl.lru(conv_integer(dci.maddress(OFFSET_HIGH downto OFFSET_LOW))), lock(0 to DSETS-1));
                  --BSM:Step2a: when lrr =>
                    --BSM:Step2a: v.setrepl := (others => '0');
                    --BSM:Step2a: if DCLOCK_BIT = 1 then 
                      --BSM:Step2a: if lock(0) = '1' then v.setrepl(0) := '1';
                      --BSM:Step2a: else
                        --BSM:Step2: v.setrepl(0) := dcramov.dtramout(0).lrr xor dcramov.dtramout(1).lrr;
                        --BSM:Step2a: v.setrepl(0) := '0';
                      --BSM:Step2a: end if;
                    --BSM:Step2a: else
                      --BSM:Step2: v.setrepl(0) := dcramov.dtramout(0).lrr xor dcramov.dtramout(1).lrr;
                      --BSM:Step2a: v.setrepl(0) := '0';
                    --BSM:Step2a: end if;
                    --BSM:Step2a: if v.setrepl(0) = '0' then
                      --BSM:Step2: v.lrr := not dcramov.dtramout(0).lrr;
                      --BSM:Step2a: v.lrr := '1';
                    --BSM:Step2a: else
                      --BSM:Step2: v.lrr := dcramov.dtramout(0).lrr;
                      --BSM:Step2a: v.lrr := '0';
                    --BSM:Step2a: end if;
                  --BSM:Step2a: end case;
                --BSM:Step2a: end if;
    
                --BSM:Step2a: if (DCLOCK_BIT = 1) then
                  --BSM:Step2a: if (hit and (not dparerr(set)) and lock(set)) = '1' then
                    --BSM:Step2a: v.lock := '1';
                  --BSM:Step2a: else
                    --BSM:Step2a: v.lock := '0';
                  --BSM:Step2a: end if;
                --BSM:Step2a: end if;
                  
              --BSM:Step2a: end if;
    
            end case;
            end if;
              
        when "001" => 		-- read miss, wait for memory data
          --BSM:Step2a: taddr := r.xaddress(OFFSET_HIGH downto LINE_LOW);
          --BSM:Step2a: newtag := r.xaddress(TAG_HIGH downto TAG_LOW);
            v.nomds := r.nomds and not eholdn;
            v.holdn := v.nomds;
            rdatasel := memory;
          
          --BSM:Step2a: for i in 0 to DSETS-1 loop
            --BSM:Step2a: wlock(i) := r.lock;
          --BSM:Step2a: end loop;
          
          --BSM:Step2a: for i in 0 to 1 loop
            --BSM:Step2a: wlrr(i) := r.lrr;
          --BSM:Step2a: end loop;
          
            if r.stpend = '0' then
                if mcdo.ready = '1' then
                    mds := r.holdn or r.nomds;
                    v.xaddress(2) := '1';
                    v.holdn := '1';
              --BSM:Step2a: if (mcdo.dcs = "01") then 
    	        --BSM:Step2a: v.hit := mcdo.cache and r.hit; twrite := v.hit;
              --BSM:Step2a: elsif (mcdo.dcs(1) = '1') then 
    	        --BSM:Step2a: v.hit := mcdo.cache and (r.hit or not r.asi(2)); twrite := v.hit;
    	      --BSM:Step2a: end if; 
              --BSM:Step2a: dwrite := twrite;
                    rdatasel := memory;
                    mexc := mcdo.mexc;
    
    	            if r.req = '0' then
    	                if (((dci.enaddr and not mds) = '1') or 
                        ((dci.eenaddr and mds and eholdn) = '1')) and (mcdo.dcs(0) = '1') then
    	                    v.dstate := "011";
                            v.holdn := '0';
    	                else 
                            v.dstate := "000";
                        end if;
    	            else
                        v.nomds := '1';
                    end if;
                end if;
    	        v.mexc := mcdo.mexc;
                v.wb.data2 := mcdo.data;
            else
        	    if ((mcdo.ready and not r.req) = '1') then	-- wait for store queue
        	        v.burst := r.size(1) and r.size(0) and not r.xaddress(2);
        	        v.wb.addr := r.xaddress;
                    v.wb.size := r.size; 
        	        v.wb.read := r.read;
                    v.wb.data1 := dci.maddress;
                    v.req := '1'; 
        	        v.wb.lock := dci.lock;
                    v.wb.asi := r.asi; 
                end if;
            end if;
        
        when "011" =>		-- return from read miss with load pending
          --BSM:Step2a: taddr := dci.maddress(OFFSET_HIGH downto LINE_LOW);
            v.dstate := "000"; 
        
        when "100" => 		-- second part of double store cycle
            v.dstate := "000";
            v.wb.data2 := dci.edata; 
            edata := dci.edata;  -- needed for STD store hit
          --BSM:Step2a: taddr := r.xaddress(OFFSET_HIGH downto LINE_LOW); 
          --BSM:Step2a: if (mcdo.dcs(0) = '1') and (r.hit = '1') then
            --BSM:Step2a: dwrite := '1';
          --BSM:Step2a: end if;
    
        when "101" =>		-- icache diag access
            rdatasel := icache;
            v.icenable := '1';
            v.holdn := '0';
            if  ico.diagrdy = '1' then
    	        v.dstate := "011";
                v.icenable := '0';
                mds := not r.read;
            end if;
    
        when "110" => 		-- wait for store buffer to empty (store access)
            edata := dci.edata;  -- needed for STD store hit
            if ((mcdo.ready and not r.req) = '1') then	-- store queue emptied
    
    	    --BSM:Step2a: if (mcdo.dcs(0) = '1') and (r.hit = '1') and (r.size = "11") then  -- write hit
              --BSM:Step2a: taddr := r.xaddress(OFFSET_HIGH downto LINE_LOW); dwrite := '1';
    	    --BSM:Step2a: end if;
                v.dstate := "000"; 
    
    	        v.req := '1';
                v.burst := r.size(1) and r.size(0);
                v.stpend := '1';
    
    	        v.wb.addr := r.xaddress;
                v.wb.size := r.size;
    	        v.wb.read := r.read;
                v.wb.data1 := dci.maddress;
    	        v.wb.lock := dci.lock;
                v.wb.data2 := dci.edata;
    	        v.wb.asi := r.asi; 
    	        if r.size = "11" then
                    v.wb.addr(2) := '0';
                end if;
            else  -- hold cpu until buffer empty
                v.holdn := '0';
            end if;
        
        when others => v.dstate := "000";
        
    end case;

    dsudata := (others => '0');
    if DEBUG_UNIT and dci.dsuen = '1' then
      --BSM:Step2a: if (DSETS > 1) then
-- pragma translate_off
        --BSM:Step2a: if not is_x(r.xaddress) then
-- pragma translate_on          
          --BSM:Step2a: v.dsuset := r.xaddress(TAG_LOW+SETBITS-1 downto TAG_LOW);

-- pragma translate_off          
        --BSM:Step2a: end if;
        --BSM:Step2a: if not is_x(r.dsuset) then
-- pragma translate_on          
        --BSM:Step2a: rdsuset := conv_integer(r.dsuset);
-- pragma translate_off          
        --BSM:Step2a: end if;
-- pragma translate_on          

      --BSM:Step2a: end if;
        case dci.asi(3 downto 0) is
            when ASI_ITAG | ASI_IDATA =>		-- Read/write Icache tags
    	        v.icenable := not ico.diagrdy;
                dsudata := ico.diagdata;
          --BSM:Step2a: when ASI_DTAG  => 
    	    --BSM:Step2a: if dci.write = '1' then 
    	        --BSM:Step2a: twrite := not dci.eenaddr; tdiagwrite := '1';
            --BSM:Step2a: end if;
            --BSM:Step2: dsudata(TAG_HIGH downto TAG_LOW) := dcramov.dtramout(rdsuset).tag;
            --BSM:Step2a: dsudata(TAG_HIGH downto TAG_LOW) := (others => '0');
            --BSM:Step2: dsudata(DLINE_SIZE -1 downto 0)  := dcramov.dtramout(rdsuset).valid;
            --BSM:Step2a: dsudata(DLINE_SIZE -1 downto 0)  := (others => '0');
            --BSM:Step2: dsudata(DCTAG_LRRPOS)  := dcramov.dtramout(rdsuset).lrr;
            --BSM:Step2a: dsudata(DCTAG_LRRPOS)  := '0';
            --BSM:Step2: dsudata(DCTAG_LOCKPOS) := dcramov.dtramout(rdsuset).lock;
            --BSM:Step2a: dsudata(DCTAG_LOCKPOS) := '0';
    
          --BSM:Step2a: when ASI_DDATA =>
    	--if (dci.write and r.dsuwren) = '1' then dwrite := '1'; ddiagwrite := '1'; end if;
            --BSM:Step2a: if dci.write = '1' then
              --BSM:Step2a: dwrite := not dci.eenaddr; ddiagwrite := '1';
            --BSM:Step2a: end if;
            --BSM:Step2: dsudata := dcramov.ddramout(rdsuset).data;
            --BSM:Step2a: dsudata := (others => '0');
            when others =>
        end case;
    end if;

-- select data to return on read access
-- align if byte/half word read from cache or memory.

    rdata := (others => '0');
    align_data := (others => '0');
    maddrlow := maddress(1 downto 0); -- stupid Synopsys VSS bug ...

    case rdatasel is
        --BSM:Step2a: when dtag => 
          --BSM:Step2: rdata(TAG_HIGH downto TAG_LOW) := dcramov.dtramout(set).tag;
          --BSM:Step2a: rdata(TAG_HIGH downto TAG_LOW) := (others => '0');
          --BSM:Step2: rdata(DLINE_SIZE -1 downto 0) := dcramov.dtramout(set).valid;
          --BSM:Step2a: rdata(DLINE_SIZE -1 downto 0) := (others => '0');
          --BSM:Step2: rdata(DCTAG_LRRPOS)  := dcramov.dtramout(set).lrr;
          --BSM:Step2a: rdata(DCTAG_LRRPOS)  := '0';
          --BSM:Step2: rdata(DCTAG_LOCKPOS) := dcramov.dtramout(set).lock;
          --BSM:Step2a: rdata(DCTAG_LOCKPOS) := '0';
    
    
        when icache => rdata := ico.diagdata;       
        when ddata | memory =>
          --BSM:Step2a: if rdatasel = ddata then 
            --BSM:Step2: align_data := dcramov.ddramout(set).data;
            --BSM:Step2a: align_data := (others => '0');
          --BSM:Step2a: else
            align_data := mcdo.data;
          --BSM:Step2a: end if;
            case size is
                when "00" => 			-- byte read
                    case maddrlow is
        	            when "00" => 
        	                rdata(7 downto 0) := align_data(31 downto 24);
        	                if signed = '1' then
                                rdata(31 downto 8) := (others => align_data(31));
                            end if;
        	            when "01" => 
        	                rdata(7 downto 0) := align_data(23 downto 16);
        	                if signed = '1' then
                                rdata(31 downto 8) := (others => align_data(23));
                            end if;
        	            when "10" => 
        	                rdata(7 downto 0) := align_data(15 downto 8);
        	                if signed = '1' then
                                rdata(31 downto 8) := (others => align_data(15));
                            end if;
        	            when others => 
        	                rdata(7 downto 0) := align_data(7 downto 0);
        	                if signed = '1' then
                                rdata(31 downto 8) := (others => align_data(7));
                            end if;
                    end case;
                when "01" => 			-- half-word read
                    if maddress(1) = '1' then 
            	        rdata(15 downto 0) := align_data(15 downto 0);
            	        if signed = '1' then
                            rdata(31 downto 15) := (others => align_data(15));
                        end if;
            	    else
            	        rdata(15 downto 0) := align_data(31 downto 16);
            	        if signed = '1' then
                            rdata(31 downto 15) := (others => align_data(31));
                        end if;
            	    end if;
                when others => 			-- single and double word read
    	            rdata := align_data;
            end case;
        when others =>
    end case;

-- select which data to update the data cache with

    --BSM:Step2a: case size is		-- merge data during partial write
        --BSM:Step2a: when "00" =>
          --BSM:Step2a: case maddrlow is
              --BSM:Step2a: when "00" =>
        	--BSM:Step2: ddatain := edata(7 downto 0) & dcramov.ddramout(set).data(23 downto 0);
            --BSM:Step2a: ddatain := (others => '0');
              --BSM:Step2a: when "01" =>
        	--BSM:Step2: ddatain := dcramov.ddramout(set).data(31 downto 24) & edata(7 downto 0) & 
        		     --BSM:Step2: dcramov.ddramout(set).data(15 downto 0);
             --BSM:Step2a: ddatain := (others => '0');        
              --BSM:Step2a: when "10" =>
        	--BSM:Step2: ddatain := dcramov.ddramout(set).data(31 downto 16) & edata(7 downto 0) & 
        		     --BSM:Step2: dcramov.ddramout(set).data(7 downto 0);
             --BSM:Step2a: ddatain := (others => '0');
              --BSM:Step2a: when others =>
        	--BSM:Step2: ddatain := dcramov.ddramout(set).data(31 downto 8) & edata(7 downto 0); 
             --BSM:Step2a: ddatain := (others => '0');
              --BSM:Step2a: end case;
        --BSM:Step2a: when "01" =>
              --BSM:Step2a: if maddress(1) = '0' then
                --BSM:Step2: ddatain := edata(15 downto 0) & dcramov.ddramout(set).data(15 downto 0);
                --BSM:Step2a: ddatain := (others => '0');
              --BSM:Step2a: else
                --BSM:Step2: ddatain := dcramov.ddramout(set).data(31 downto 16) & edata(15 downto 0);
                --BSM:Step2a: ddatain := (others => '0');
              --BSM:Step2a: end if;
        --BSM:Step2a: when others => 
              --BSM:Step2a: ddatain := edata;
    --BSM:Step2a: end case;

-- handle double load with pipeline hold

    if (r.dstate = "000") and (r.nomds = '1') then
        rdata := r.wb.data2;
        mexc := r.mexc;
    end if;

-- Handle AHB retry. Re-generate bus request and burst

    if mcdo.retry = '1' then
        v.req := '1';
        v.burst := r.wb.size(0) and r.wb.size(1) and not r.wb.addr(2);
    end if;

-- Generate new valid bits

    --BSM:Step2a: vmaskdbl := decode(maddress(LINE_HIGH downto LINE_LOW+1));
    --BSM:Step2a: if (size = "11") and (read = '0') then 
      --BSM:Step2a: for i in 0 to (DLINE_SIZE - 1) loop
        --BSM:Step2a: vmaskraw(i) := vmaskdbl(i/2);
      --BSM:Step2a: end loop;
    --BSM:Step2a: else
      --BSM:Step2a: vmaskraw := decode(maddress(LINE_HIGH downto LINE_LOW));
    --BSM:Step2a: end if;

    --BSM:Step2a: vmask := vmaskraw;
    --BSM:Step2a: if r.hit = '1' then
        --BSM:Step2a: vmask := r.valid or vmaskraw;
    --BSM:Step2a: end if;
    --BSM:Step2a: if r.dstate = "000" then 
      --BSM:Step2: vmask := dcramov.dtramout(set).valid or vmaskraw;
      --BSM:Step2a: vmask := vmaskraw;
    --BSM:Step2a: end if;

    --BSM:Step2a: if (mcdo.mexc or r.flush) = '1' then
        --BSM:Step2a: twrite := '0'; dwrite := '0';
    --BSM:Step2a: end if;
    --BSM:Step2a: if twrite = '1' then
      --BSM:Step2a: v.valid := vmask;
      --BSM:Step2a: if (DSETS>1) and (DCREPLACE = lru) and (tdiagwrite = '0') then
        --BSM:Step2a: vl.write := '1'; vl.set := setrepl;
      --BSM:Step2a: end if;
    --BSM:Step2a: end if;

    --BSM:Step2a: if (DSETS>1) and (DCREPLACE = lru) and (rl.write = '1') then
      --BSM:Step2a: vl.lru(conv_integer(rl.waddr)) :=
      --BSM:Step2a: lru_calc(rl.lru(conv_integer(rl.waddr)), conv_integer(rl.set));
    --BSM:Step2a: end if;

    --BSM:Step2a: if tdiagwrite = '1' then -- diagnostic tag write
      --BSM:Step2a: if DEBUG_UNIT and (dci.dsuen = '1') then
        --BSM:Step2a: vmask := dci.maddress(DLINE_SIZE - 1 downto 0);
      --BSM:Step2a: else
        --BSM:Step2a: vmask := dci.edata(DLINE_SIZE - 1 downto 0);
        --BSM:Step2a: newtag(TAG_HIGH downto TAG_LOW) := dci.edata(TAG_HIGH downto TAG_LOW);
        --BSM:Step2a: for i in 0 to 1 loop
            --BSM:Step2a: wlrr(i)  := dci.edata(DCTAG_LRRPOS);
        --BSM:Step2a: end loop;
        --BSM:Step2a: for i in 0 to DSETS-1 loop
            --BSM:Step2a: wlock(i) := dci.edata(DCTAG_LOCKPOS);
        --BSM:Step2a: end loop;
      --BSM:Step2a: end if;
    --BSM:Step2a: end if;

-- cache flush

    --BSM:Step2a: if (dci.flush or flush or mcdo.dflush) = '1' then
      --BSM:Step2a: v.flush := '1'; v.faddr := (others => '0');
    --BSM:Step2a: end if;

    --BSM:Step2a: if r.flush = '1' then
      --BSM:Step2a: twrite := '1';
      --BSM:Step2a: vmask := (others => '0');
      --BSM:Step2a: v.faddr := r.faddr +1; 
      --BSM:Step2a: newtag(TAG_HIGH downto TAG_LOW) := (others => '0');
      --BSM:Step2a: taddr(OFFSET_HIGH downto OFFSET_LOW) := r.faddr;
      --BSM:Step2a: wlrr := (others => '0');
      --BSM:Step2a: if (r.faddr(DOFFSET_BITS -1) and not v.faddr(DOFFSET_BITS -1)) = '1' then
	    --BSM:Step2a: v.flush := '0';
      --BSM:Step2a: end if;
    --BSM:Step2a: end if;

-- AHB snoop handling (2), bypass write data on read/write contention

    --BSM:Step2b: if DSNOOP then
-- pragma translate_off
      --BSM:Step2a: if not is_x(setrepl) then
-- pragma translate_on
        --BSM:Step2a: if tdiagwrite = '1' then
            --BSM:Step2a: snoopset2 := set; 
	    --BSM:Step2a: else
            --BSM:Step2a: snoopset2 := conv_integer(setrepl); end if;
-- pragma translate_off
      --BSM:Step2a: end if;
-- pragma translate_on
        --BSM:Step2b: if DSNOOP_FAST then
            --BSM:Step2a: vh.taddr := taddr(OFFSET_HIGH downto OFFSET_LOW);
            --BSM:Step2b: vh.taddr := (others => '0');
            --BSM:Step2b: vh.set := std_logic_vector(conv_unsigned(set, SETBITS));
	    --BSM:Step2a: if twrite = '1' then 
-- pragma translate_off
          --BSM:Step2a: if not is_x(taddr(OFFSET_HIGH downto OFFSET_LOW)) then
-- pragma translate_on
	        --BSM:Step2a: vh.hit(conv_integer(taddr(OFFSET_HIGH downto OFFSET_LOW)))(snoopset2) := '0';
-- pragma translate_off
          --BSM:Step2a: end if;
-- pragma translate_on
	    --BSM:Step2a: end if;
      --BSM:Step2a: else
        --BSM:Step2a: if rs.addr(OFFSET_HIGH  downto OFFSET_LOW) = r.xaddress(OFFSET_HIGH  downto OFFSET_LOW) then 
	        --BSM:Step2a: if twrite = '0' then 
                --BSM:Step2a: if snoopwe = '1'
                    --BSM:Step2a: then vs.writebp(snoopset) := '1';
                --BSM:Step2a: end if;
	        --BSM:Step2a: else
                --BSM:Step2a: if snoopwe = '1' then
                    --BSM:Step2a: twrite := '0';
                --BSM:Step2a: end if; -- avoid write/write contention
	        --BSM:Step2a: end if;
	    --BSM:Step2a: end if;
        --BSM:Step2b: end if;
    --BSM:Step2b: end if;

-- update cache with memory data during read miss

    --BSM:Step2a: if read = '1' then
        --BSM:Step2a: ddatain := mcdo.data;
    --BSM:Step2a: end if;

-- cache write signals

    --BSM:Step2a: if twrite = '1' then
      --BSM:Step2a: if tdiagwrite = '1' then
        --BSM:Step2a: ctwrite(set) := '1';
      --BSM:Step2a: else
        --BSM:Step2a: ctwrite(conv_integer(setrepl)) := '1';
      --BSM:Step2a: end if;
    --BSM:Step2a: end if;
    --BSM:Step2a: if dwrite = '1' then
      --BSM:Step2a: if ddiagwrite = '1' then
        --BSM:Step2a: cdwrite(set) := '1';
      --BSM:Step2a: else
        --BSM:Step2a: cdwrite(conv_integer(setrepl)) := '1';
      --BSM:Step2a: end if;
    --BSM:Step2a: end if;
      
    --BSM:Step2c: csnoopwe := (others => '0');
    --BSM:Step2c: if (snoopwe = '1') then
        --BSM:Step2c: csnoopwe(snoopset) := '1';
    --BSM:Step2c: end if;

    --BSM:Step2a: if (r.flush and twrite) = '1' then   -- flush 
       --BSM:Step2a: ctwrite := (others => '1'); 
       --BSM:Step2a: wlrr := (others => '0');
       --BSM:Step2a: wlock := (others => '0');
    --BSM:Step2a: end if;

    --BSM:Step2a: if (r.flush or (not rst)) = '1' then
      --BSM:Step2a: vl.lru := (others => (others => '0'));
    --BSM:Step2a: end if;
    
    
-- reset

    if rst = '0' then 
        v.dstate := "000";
        v.stpend  := '0';
        v.req := '0';
        v.burst := '0';
        v.read := '0';
        v.flush := '0';
        v.nomds := '0';
        --BSM:Step2b: v.rndcnt := (others => '0');
        --BSM:Step2b: v.setrepl := (others => '0');
        --BSM:Step2c: v.dsuset := (others => '0');
      --BSM:Step2a: v.lrr := '0';
        v.lock := '0';
    end if;


-- Drive signals -- register inputs
    c <= v;
    --BSM:Step2b: cs <= vs;
    --BSM:Step2b: ch <= vh; 
    --BSM:Step2a: cl <= vl;

    
    -- tag ram inputs
    --BSM:Step2: dcrami.dtramin.valid    <= vmask;
    --BSM:Step2: dcrami.dtramin.tag      <= newtag(TAG_HIGH downto TAG_LOW);
    --BSM:Step2: dcrami.dtramin.lrr      <= wlrr;
    --BSM:Step2: dcrami.dtramin.lock     <= wlock;
    --BSM:Step2: dcrami.dtramin.enable   <= enable;
    --BSM:Step2: dcrami.dtramin.write    <= ctwrite;
    --BSM:Step2: dcrami.dtraminsn.enable <= vs.snoop or snoopwe;
    --BSM:Step2: dcrami.dtraminsn.write  <= csnoopwe;
    --BSM:Step2: dcrami.dtraminsn.address<= snoopaddr;
    --BSM:Step2: dcrami.dtraminsn.tag    <= rs.addr(TAG_HIGH downto TAG_LOW);
    
    -- data ram inputs
    --BSM:Step2: dcrami.ddramin.enable   <= enable;
    --BSM:Step2: dcrami.ddramin.address  <= taddr;
    --BSM:Step2: dcrami.ddramin.data     <= ddatain;
    --BSM:Step2: dcrami.ddramin.write    <= cdwrite;

    -- memory controller inputs
    mcdi.address  <= r.wb.addr;
    mcdi.data     <= r.wb.data1;
    mcdi.burst    <= r.burst;
    mcdi.size     <= r.wb.size;
    mcdi.read     <= r.wb.read;
    mcdi.asi      <= r.wb.asi;
    mcdi.lock     <= r.wb.lock or dci.lock;
    mcdi.req      <= r.req;
    mcdi.flush    <= r.flush;

    -- diagnostic instruction cache access
    dco.icdiag.flush  <= iflush or mcdo.iflush;
    dco.icdiag.read   <= read;
    dco.icdiag.tag    <= not r.asi(0);
    dco.icdiag.addr   <= r.xaddress;
    dco.icdiag.enable <= r.icenable;
    dco.dsudata       <= dsudata;	-- debug unit
 
    -- IU data cache inputs
    dco.data  <= rdata;
    dco.mexc  <= mexc;
    dco.hold  <= r.holdn;
    dco.mds   <= mds;
    dco.werr  <= mcdo.werr;
    
    end process;

-- Local registers
    reg1 : process(clk)
    begin
        if rising_edge(clk ) then
            r <= c;
        end if;
        
    end process;
    --BSM:Step2b: sn2 : if DSNOOP generate
      --BSM:Step2b: reg2 : process(clk)
      --BSM:Step2b: begin if rising_edge(clk ) then rs <= cs; end if; end process;
    --BSM:Step2b: end generate;
    --BSM:Step2b: sn3 : if DSNOOP_FAST generate
      --BSM:Step2b: reg3 : process(clk)
      --BSM:Step2b: begin if rising_edge(clk ) then rh <= ch; end if; end process;
    --BSM:Step2b: end generate;

    --BSM:Step2: reg2 : if (DSETS>1) and (DCREPLACE = lru) generate
      --BSM:Step2: reg2 : process(clk)
      --BSM:Step2: begin if rising_edge(clk ) then rl <= cl; end if; end process;
    --BSM:Step2: end generate;   

    
-- pragma translate_off
  --BSM:Step2b: chk : process
  --BSM:Step2b: begin
    --BSM:Step2b: assert not ((DSETS > 2) and (DCREPLACE = lrr)) report
	--BSM:Step2b: "Wrong data cache configuration detected: LRR replacement requires 2 sets"
    --BSM:Step2b: severity failure;
    --BSM:Step2b: wait;
  --BSM:Step2b: end process;
-- pragma translate_on

end ;

