-------------------------------------------------------------------------------
--  This file is a part of the LEON VHDL model
--  Copyright (C) 1999  European Space Agency (ESA)
--
--  This library is free software; you can redistribute it and/or
--  modify it under the terms of the GNU Lesser General Public
--  License as published by the Free Software Foundation; either
--  version 2 of the License, or (at your option) any later version.
--
--  See the file COPYING.LGPL for the full details of the license.

-------------------------------------------------------------------------------
-- Entity: 	leon
-- File:	leon.vhd
-- Author:	Jiri Gaisler - ESA/ESTEC
-- Description:	Complete processor
-------------------------------------------------------------------------------
-- Modified:  B McCabe 30.06.2003
-- 1.Removed commented legacy code used in APEX build and NINJA2
-- 2.Removed pad arrangement in readiness for instantiating LEON as component 
--   in NINJA4 design
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use work.target.all;
use work.config.all;
use work.iface.all;
use work.tech_map.all;
use work.amba.all;
-- pragma translate_off
use work.debug.all;
-- pragma translate_on

entity leon is
    port (
        resetn   : in    std_logic; 			            -- system signals
        clk      : in    std_logic;
    
        errorn   : out   std_logic;
        address  : out   std_logic_vector(22 downto 0); 	-- memory bus
    
        data     : inout std_logic_vector(31 downto 16);
    
        brdyn    : in    std_logic;
        bexcn    : in    std_logic;
        
        -- RAM config - BSM
    	ramcs0   : out	 std_logic;
        ramwe0 	 : out	 std_logic;                         
    	ramoe0	 : out	 std_logic;
    	rambe0	 : out	 std_logic;
    	rambe1   : out	 std_logic;
        rambe2   : out	 std_logic;
        rambe3   : out	 std_logic;
        
        -- Flash config - BSM
        romcs    : out   std_logic;
        romwe    : out   std_logic;
        romoe    : out   std_logic;
    
        pio      : inout std_logic_vector(15 downto 0); 	-- I/O port
    
        wdogn    : out   std_logic;				            -- watchdog output
    
        dsuen    : in    std_logic;
        dsutx    : out   std_logic;
        dsurx    : in    std_logic;
        dsubre   : in    std_logic;
        dsuact   : out   std_logic;
        test     : in    std_logic;
        
        ext_irq  : in    std_logic_vector(1 downto 0);
        
        ext_apbi : out   apb_slv_in_vector(0 to 3); 
        ext_apbo : in    apb_slv_out_vector(0 to 3)
    );
end; 

architecture rtl of leon is

component mcore
    port (
        resetn   : in    std_logic;
        clk      : in    clk_type;
        clkn     : in    clk_type;
        pciclk   : in    clk_type;
        memi     : in    memory_in_type;
        memo     : out   memory_out_type;
        ioi      : in    io_in_type;
        ioo      : out   io_out_type;
        pcii     : in    pci_in_type;
        pcio     : out   pci_out_type;
        dsi      : in    dsuif_in_type;
        dso      : out   dsuif_out_type;
        sdo      : out   sdram_out_type;
        
        ext_irq  : in    std_logic_vector(1 downto 0);
        ext_apbi : out   apb_slv_in_vector(0 to 3);
        ext_apbo : in    apb_slv_out_vector(0 to 3);

        test     : in    std_logic
);
end component; 

signal gnd, vcc, clko, sdclkl, resetno  : std_logic;
signal clkm, clkn, pciclk               : clk_type;
signal memi                             : memory_in_type;
signal memo                             : memory_out_type;
signal ioi                              : io_in_type;
signal ioo                              : io_out_type;
signal pcii                             : pci_in_type;
signal pcio                             : pci_out_type;
signal dsi                              : dsuif_in_type;
signal dso                              : dsuif_out_type;
signal sdo                              : sdram_out_type;
signal pllref                           : clk_type;
signal pllctrl                          : std_logic_vector(1 downto 0);
signal enable                           : std_logic;

begin
    gnd <= '0';
	vcc <= '1';
    enable <= '1';
  
-- main processor core
    mcore0  : mcore  
        port map ( 
            resetn => resetno,
            clk => clkm,
            clkn => clkn,
            pciclk => pciclk,
            memi => memi,
            memo => memo,
            ioi => ioi,
            ioo => ioo,
            pcii => pcii,
            pcio => pcio,
            dsi => dsi,
            dso => dso,
            sdo => sdo,
            ext_irq => ext_irq,
            ext_apbi => ext_apbi,
            ext_apbo => ext_apbo,
            test => test
    );

-- clock generator
    clkgen0 : clkgen
        port map (
            clkin => clko,   
            pciclkin => clko,
            clk => clkm,     
            clkn => clkn,    
            sdclk => sdclkl,   
            pciclk => pciclk,  
            pllref => pllref,  
            pllctrl => pllctrl 
    );
    
    clko <= clk;					
    resetno <= resetn;
    
    memi.brdyn <= '1';--brdyn;     -- tie HIGH in Fm_Cpu
    memi.bexcn <= '1';--bexcn;     -- tie HIGH in Fm_Cpu

    errorn <= not ioo.errorn;
    
    d : for i in 16 to 31 generate			-- data bus
        data(i) <= memo.data(i) when memo.bdrive((31-i)/8) = '0' else 'Z';
        memi.data(i) <= data(i);
    end generate;

    io : for i in 0 to 15 generate		        -- parallel I/O port
        pio(i) <= ioo.piol(i) when ioo.piodir(i) = '0' else 'Z';
        ioi.piol(i) <= pio(i);
    end generate;

    a : for i in 0 to 22 generate			-- memory address
        address(i) <= memo.address(i);
    end generate;

    -- RAM config - BSM
    ramwe0 <= memo.writen;
    ramoe0 <= memo.ramoen(0);
    ramcs0 <= memo.ramsn(0);
    
    rambe0 <= vcc;
	rambe1 <= vcc;
	rambe2 <= (memo.wrn(1) and memo.wrn(3) and memo.oen);
    rambe3 <= (memo.wrn(0) and memo.wrn(2) and memo.oen);
    -- end of RAM config - BSM

    -- Flash config - BSM
    romcs <= memo.romsn(0) and memo.romsn(1);
    romwe <= memo.writen;
    romoe <= memo.oen;
    -- end of ROM config - BSM

    wd : if WDOGEN generate
        wdogn <= ioo.wdog;
    end generate;

    ds : if DEBUG_UNIT generate
        dsi.dsui.dsuen <= '1';
        dsutx <= dso.dcomo.dsutx;
        dsi.dcomi.dsurx <= dsurx; 
        dsi.dsui.dsubre <= dsubre;
        dsuact <= dso.dsuo.dsuact;
    end generate;

end ;

